@@ -36,9 +36,14 @@ from solfege import soundcard
 from solfege import utils
 from solfege import winlang
 from solfege.instrumentselector import nInstrumentSelector, InstrumentConfigurator
-
 import solfege
 
+try:
+    import solfege.soundcard.alsa_sequencer
+except ImportError:
+    solfege.soundcard.alsa_sequencer = None
+
+
 if sys.platform == 'win32':
     try:
         from solfege.soundcard import winmidi
@@ -63,7 +68,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         self.set_default_size(400, 400)
         self.g_notebook = gtk.Notebook()
         self.vbox.pack_start(self.g_notebook)
-        
+
         ########
         # midi #
         ########
@@ -79,7 +84,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             gtk.Adjustment(self.get_int('config/default_bpm'), 10, 500, 1, 10))
         self.g_arpeggio_bpm = gu.nSpinButton('config', 'arpeggio_bpm',
             gtk.Adjustment(self.get_int('config/arpeggio_bpm'), 10, 500, 1, 10))
-        for text, widget in [(_("_Default:") ,self.g_default_bpm), 
+        for text, widget in [(_("_Default:") ,self.g_default_bpm),
                              (_("A_rpeggio:") ,self.g_arpeggio_bpm)]:
             label = gtk.Label(text)
             label.set_alignment(0.0, 0.5)
@@ -90,7 +95,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             label = gtk.Label(_("BPM"))
             label.set_alignment(0.0, 0.5)
             label.set_tooltip_text(_("Beats per minute"))
-            tempo_hbox.pack_start(label, expand=True)        
+            tempo_hbox.pack_start(label, expand=True)
         category_vbox.pack_start(tempo_hbox, False)
 
         box, category_vbox = gu.hig_category_vbox(_("Preferred Instrument"))
@@ -207,10 +212,10 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             self.get_string("app/wav_to_mp3_cmd"))
         self.g_mp3_convertor_options.child.set_text(
             self.get_string("app/wav_to_mp3_cmd_options"))
-        self.g_mp3_convertor.connect('changed', 
+        self.g_mp3_convertor.connect('changed',
             lambda w: self.set_string('app/wav_to_mp3_cmd',
                 w.child.get_text()))
-        self.g_mp3_convertor_options.connect('changed', 
+        self.g_mp3_convertor_options.connect('changed',
             lambda w: self.set_string('app/wav_to_mp3_cmd_options',
                 w.child.get_text()))
         # wav_to_ogg
@@ -231,10 +236,10 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             self.get_string("app/wav_to_ogg_cmd"))
         self.g_ogg_convertor_options.child.set_text(
             self.get_string("app/wav_to_ogg_cmd_options"))
-        self.g_ogg_convertor.connect('changed', 
+        self.g_ogg_convertor.connect('changed',
             lambda w: self.set_string('app/wav_to_ogg_cmd',
                 w.child.get_text()))
-        self.g_ogg_convertor_options.connect('changed', 
+        self.g_ogg_convertor_options.connect('changed',
             lambda w: self.set_string('app/wav_to_ogg_cmd_options',
                 w.child.get_text()))
 
@@ -385,7 +390,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         self.g_autorepeat_if_wrong = gu.nCheckButton('config', 'auto_repeat_question_if_wrong_answer', _("_Repeat question if the answer was wrong"))
         category_vbox.pack_start(self.g_autorepeat_if_wrong, False)
 
-        self.g_expert_mode = gu.nCheckButton('gui', 
+        self.g_expert_mode = gu.nCheckButton('gui',
                 'expert_mode', _("E_xpert mode"))
         self.g_expert_mode.connect('toggled', solfege.app.reset_exercise)
         category_vbox.pack_start(self.g_expert_mode, False)
@@ -434,7 +439,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                  # testfile is a file in lesson-files/share
                  'testfile': 'fifth-small-220.00.wav',
                  'label': _("WAV:"),
-                 # players is a list of tuples. The tuple has to or more 
+                 # players is a list of tuples. The tuple has to or more
                  # items. The first is the binary, and the rest is possible
                  # sets of command line options that might work.
                  # '/path/to/player', 'comandline', '
@@ -474,7 +479,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
              'players': [
                         ('gst-launch', 'playbin uri=file://%s',
                                        'filesrc location=%s ! oggdemux ! vorbisdec ! audioconvert ! alsasink'),
-                        ('ogg123', ''), 
+                        ('ogg123', ''),
                         ('alsaplayer', ''),
              ],
             }
@@ -493,7 +498,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                 if s[0] == cmd_combo.child.get_text():
                     for o in s[1:]:
                         cmd_combo.opts.append_text(o)
-            
+
             def _changed(widget, formatid):
                 if widget.get_active() != -1:
                     widget.opts.get_model().clear()
@@ -539,9 +544,36 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         self.g_fakesynth_radio = gu.RadioButton(None, _("_No sound"), None)
         category_vbox.pack_start(self.g_fakesynth_radio, False)
 
+        ### ALSA
+        hbox = gu.bHBox(category_vbox, False)
+        self.g_alsa_radio = gu.RadioButton(self.g_fakesynth_radio,
+              _("Use ALSA _device"), None)
+        hbox.pack_start(self.g_alsa_radio, False)
+
+        self.g_alsa_device = gtk.ComboBox()
+        cell = gtk.CellRendererText()
+        self.g_alsa_device.pack_start(cell, True)
+        self.g_alsa_device.add_attribute(cell, 'text', 0)
+        hbox.pack_start(self.g_alsa_device, False)
+        if solfege.soundcard.alsa_sequencer:
+            liststore = gtk.ListStore(gobject.TYPE_STRING)
+            v = self.get_list("sound/alsa-client-port")
+            for clientid, portid, clientname, portname in soundcard.alsa_sequencer.get_connection_list():
+                liststore.append(("%i:%i %s %s" % (clientid, portid, clientname, portname),))
+            self.g_alsa_device.set_model(liststore)
+            for idx, v in enumerate(solfege.soundcard.alsa_sequencer.get_connection_list()):
+                if v[0:2] == self.get_list("sound/alsa-client-port"):
+                    self.g_alsa_device.set_active(idx)
+        else:
+            self.g_alsa_device.set_sensitive(False)
+            self.g_alsa_radio.set_sensitive(False)
+            label = gtk.Label("Disabled because the pyalsa Python module was not found.")
+            label.show()
+            hbox.pack_start(label, False)
+        ### OSS
         hbox = gu.bHBox(category_vbox, False)
         self.g_device_radio = gu.RadioButton(self.g_fakesynth_radio,
-              _("Use _device"), None)
+              _("Use OSS _device"), None)
         hbox.pack_start(self.g_device_radio, False)
 
         liststore = gtk.ListStore(gobject.TYPE_STRING)
@@ -565,6 +597,8 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             self.g_midiplayer_radio.set_active(True)
         elif self.get_string("sound/type") == "sequencer-device":
             self.g_device_radio.set_active(True)
+        elif self.get_string("sound/type") == "alsa-sequencer":
+            self.g_alsa_radio.set_active(True)
         else:
             self.g_fakesynth_radio.set_active(True)
 
@@ -623,10 +657,20 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             self.g_fakesynth_radio.set_active(True)
         elif self.get_string("sound/type") == "external-midiplayer":
             self.g_midiplayer_radio.set_active(True)
+        elif self.get_string("sound/type") == "alsa-sequencer":
+            if solfege.soundcard.alsa_sequencer:
+                self.g_alsa_radio.set_active(True)
+                p = self.get_list("sound/alsa-client-port")
+                for idx, k in enumerate(solfege.soundcard.alsa_sequencer.get_connection_list()):
+                    if p[0] == k[0] and p[1] == k[1]:
+                        self.g_alsa_device.set_active(idx)
+                        break
+            else:
+                self.set_string("sound/type", "fake-synth")
+                self.g_fakesynth_radio.set_active(True)
         else:
             assert self.get_string("sound/type") in ("winsynth", "sequencer-device")
             self.g_device_radio.set_active(True)
-        return -1
     def apply_and_close(self, w, response):
         if response ==  gtk.RESPONSE_DELETE_EVENT:
             self.set_gui_from_config()
@@ -676,6 +720,9 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             except (soundcard.SoundInitException, OSError, ImportError), e:
                 solfege.app.display_sound_init_error_message(e)
                 return -1
+        elif solfege.soundcard.alsa_sequencer and self.g_alsa_radio.get_active():
+            v = solfege.soundcard.alsa_sequencer.get_connection_list()[self.g_alsa_device.get_active()]
+            soundcard.initialise_alsa_sequencer(v[0:2])
         else: # no sound
             assert self.g_fakesynth_radio.get_active()
             soundcard.initialise_using_fake_synth(0)
@@ -686,12 +733,16 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                 self.set_string("sound/type", "winsynth")
             else:
                 self.set_string("sound/type", "sequencer-device")
+        elif solfege.soundcard.alsa_sequencer and self.g_alsa_radio.get_active():
+            self.set_string("sound/type", "alsa-sequencer")
+            v = solfege.soundcard.alsa_sequencer.get_connection_list()[self.g_alsa_device.get_active()]
+            self.set_list("sound/alsa-client-port", v[0:2])
         else:
             assert self.g_fakesynth_radio.get_active()
             self.set_string("sound/type", "fake-synth")
         if sys.platform != 'win32':
             self.set_string("sound/device_file", self.g_device_file.child.get_text())
-        if soundcard.synth.m_type_major not in ('Midifile', 'Fake'):
+        if soundcard.synth.m_type_major in ('OSS', 'win32'):
             self.set_int("sound/synth_number", soundcard.synth.m_devnum)
             # we set the spin just in case m_devnum was changed by the
             # soundcard setup code, if it was out of range
@@ -717,7 +768,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         if gu.dialog_delete(_("Delete obsolete statistics?"), self,
                 _("This will delete the directory «%s».") % os.path.join(filesystem.app_data(), u"statistics")):
             try:
-                shutil.rmtree(os.path.join(filesystem.app_data(), u"statistics"))   
+                shutil.rmtree(os.path.join(filesystem.app_data(), u"statistics"))
             except OSError, e:
                 gu.display_exception_message(e)
             self.update_old_statistics_info()
